#include <gtest/gtest.h>
#include <cmath>
#include "generators.hpp"
#include "../classes/preprocessing/accuracyestimator.hpp"

TEST(AccuracyEstimatorTest, TooSmallDifferenceTest)
{
    AccuracyEstimator estimator;
    std::vector<double_t> input {0.0, 1e-17, 5.9, -10.2, 3.4, 3.5};
    EXPECT_NEAR(0.1, estimator.computeAccuracy(input), 1e-10);
}

TEST(AccuracyEstimatorTest, AccuracyAtLowestValueTest)
{
    AccuracyEstimator estimator;
    std::vector<double_t> input {-10.19, 0.0, 5.9, -10.2, 3.4, 3.5};
    EXPECT_NEAR(0.01, estimator.computeAccuracy(input), 1e-10);
}

TEST(AccuracyEstimatorTest, AccuracyAtHighestValueTest)
{
    AccuracyEstimator estimator;
    std::vector<double_t> input {0.0, 5.95, 5.9, -10.2, 3.4, 3.5};
    EXPECT_NEAR(0.05, estimator.computeAccuracy(input), 1e-10);
}

TEST(AccuracyEstimatorTest, SetAccuracyOfVariableTest)
{
    AccuracyEstimator estimator;

    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"});
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num");
    std::unique_ptr<EventLog> log = Generators::generateEventLog(catVariable, numVariable);

    numVariable->setAccuracy(42.0);
    EXPECT_NEAR(42.0, numVariable->getAccuracy(), 1e-10);
    estimator.initializeAllAccuracies(log.get());
    EXPECT_NEAR(1.0, numVariable->getAccuracy(), 1e-10);
}
