#include <gtest/gtest.h>
#include "../classes/datamodel/conditions/condition.hpp"
#include "../classes/datamodel/conditions/oroperator.hpp"

TEST(ConditionTest, ConstructorSuccessTest)
{
    Condition::ElementsType elements;
    elements.push_back(std::make_unique<OrOperator>());
    Condition condition(elements);
    EXPECT_EQ(1, condition.getElements().size());
}

TEST(ConditionTest, ConstructorEmptySuccessTest)
{
    Condition::ElementsType elements;
    Condition condition(elements);
    EXPECT_EQ(0, condition.getElements().size());
}

TEST(ConditionTest, ConstructorExceptionTest)
{
    Condition::ElementsType elements;
    elements.push_back(std::make_unique<OrOperator>());
    elements.push_back(nullptr);
    EXPECT_THROW(Condition condition(elements), std::invalid_argument);
}

TEST(ConditionTest, ElementsSetSuccessTest)
{
    Condition::ElementsType elements;
    elements.push_back(std::make_unique<OrOperator>());
    Condition condition(elements);
    Condition::ElementsType newElements;
    condition.setElements(newElements);
    EXPECT_EQ(0, condition.getElements().size());
}

TEST(ConditionTest, ElementsSetExceptionTest)
{
    Condition::ElementsType elements;
    elements.push_back(std::make_unique<OrOperator>());
    Condition condition(elements);
    Condition::ElementsType newElements;
    newElements.push_back(nullptr);
    EXPECT_THROW(condition.setElements(newElements), std::invalid_argument);
}