#include <memory>
#include "../classes/datamodel/eventlogs/categoricalvariable.hpp"
#include "../classes/datamodel/eventlogs/numericalvariable.hpp"
#include "../classes/datamodel/eventlogs/trace.hpp"
#include "../classes/datamodel/eventlogs/eventlog.hpp"
#include "../classes/datamodel/conditions/atomiccondition.hpp"
#include "../classes/datamodel/rulemodel.hpp"

class Generators
{
    public:
    static std::unique_ptr<Trace> generateComplexTrace(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<CategoricalVariable> otherVariable = std::make_shared<CategoricalVariable>("other",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));

    static std::unique_ptr<EventLog> generateEventLog(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));
    static std::unique_ptr<EventLog> generateEventLogWithNumericalUpdateRule(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<CategoricalVariable> otherVariable = std::make_shared<CategoricalVariable>("other",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));
    static std::unique_ptr<EventLog> generateEventLogWithCategoricalUpdateRule(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));
    static std::unique_ptr<EventLog> generateEventLogWithTransitionalUpdateRule(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));
    static std::unique_ptr<EventLog> generateEventLogWithMissingValues(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));

    static std::unique_ptr<RuleModel> generateRuleModel(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));

    static std::unique_ptr<RuleModel> generateCompleteRuleModel(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));

    static std::unique_ptr<RuleModel> generateRuleModelForRuleMerging(
    std::shared_ptr<CategoricalVariable> catVariable = std::make_shared<CategoricalVariable>("vowel",
        std::set<std::string>{"a", "e", "i", "o", "u"}),
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num"));

    static std::unique_ptr<Trace> generateBelowTrace(std::shared_ptr<NumericalVariable> numVariable);
    static std::unique_ptr<Trace> generateEqualTrace(std::shared_ptr<NumericalVariable> numVariable);
    static std::unique_ptr<Trace> generateAboveTrace(std::shared_ptr<NumericalVariable> numVariable);

    static std::pair<std::unique_ptr<AtomicCondition>, std::unique_ptr<Trace>> generateTransitionData();
};
