#include <gtest/gtest.h>
#include "../classes/datamodel/updaterules/valuesetrule.hpp"
#include "../classes/datamodel/updaterules/valuerule.hpp"
#include "../classes/datamodel/conditions/valuecondition.hpp"

TEST(LogVariableTest, CategoricalVariableInRule)
{
    std::set<std::string> vowelDomain = {"a", "e", "i", "o", "u"};
    std::shared_ptr<CategoricalVariable> catVariable =
        std::make_shared<CategoricalVariable>("vowel", vowelDomain);
    ValueSetRule::SetType set = {"a", "i", "o"};
    ValueSetRule rule(catVariable, set);
    std::shared_ptr<CategoricalVariable> catVariableInRule = 
        std::dynamic_pointer_cast<CategoricalVariable>(rule.getVariable());
    EXPECT_EQ("vowel", catVariableInRule->getName());
    EXPECT_EQ(vowelDomain, catVariableInRule->getDomain());
}

TEST(LogVariableTest, NumericalVariableInRule)
{
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num");
    ValueRule rule(numVariable, 1.0);
    std::shared_ptr<NumericalVariable> numVariableInRule =
        std::dynamic_pointer_cast<NumericalVariable>(rule.getVariable());
    EXPECT_EQ("num", numVariableInRule->getName());
}

TEST(LogVariableTest, CategoricalVariableInCondition)
{
    std::set<std::string> vowelDomain = {"a", "e", "i", "o", "u"};
    std::shared_ptr<CategoricalVariable> catVariable =
        std::make_shared<CategoricalVariable>("vowel", vowelDomain);
    ValueCondition<std::string> cond(catVariable, "i");
    const std::shared_ptr<CategoricalVariable> catVariableInCondition = 
        std::static_pointer_cast<CategoricalVariable>(cond.getVariable());
    EXPECT_EQ("vowel", catVariableInCondition->getName());
    EXPECT_EQ(vowelDomain, catVariableInCondition->getDomain());
}

TEST(LogVariableTest, NumericalVariableInCondition)
{
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num");
    ValueCondition<double_t> rule(numVariable, 1.0);
    const std::shared_ptr<NumericalVariable> numVariableInRule = 
        std::static_pointer_cast<NumericalVariable>(rule.getVariable());
    EXPECT_EQ("num", numVariableInRule->getName());
}
