#include <gtest/gtest.h>
#include <stdexcept>
#include "../classes/scoring/mdlcalculator.hpp"

TEST(MdlCalculatorBinomialTest, NegativeTotalTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeBinomial(-1, 2), std::domain_error);
}

TEST(MdlCalculatorBinomialTest, ZeroTotalTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeBinomial(0, 2), std::domain_error);
}

TEST(MdlCalculatorBinomialTest, NegativeChoicesTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeBinomial(2, -1), std::domain_error);
}

TEST(MdlCalculatorBinomialTest, ChoicesLargerTotalTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeBinomial(2, 3), std::domain_error);
}

TEST(MdlCalculatorBinomialTest, ZeroChoicesTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(0.0, calc.codeBinomial(2, 0));
}

TEST(MdlCalculatorBinomialTest, ChoicesEqualsTotalTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(0.0, calc.codeBinomial(3, 3));
}

TEST(MdlCalculatorBinomialTest, ChoicesOneTest)
{
    MdlCalculator calc;
    const double_t expected = 3.0;
    const double_t abs_error = expected * std::pow(10.0, -(DBL_DIG - 1));
    EXPECT_NEAR(expected, calc.codeBinomial(8, 1), abs_error);
}

TEST(MdlCalculatorBinomialTest, SmallTest)
{
    MdlCalculator calc;
    const double_t expected = 6.129283016944966;
    const double_t abs_error = expected * std::pow(10.0, -(DBL_DIG - 1));
    EXPECT_NEAR(expected, calc.codeBinomial(8, 4), abs_error);
}

TEST(MdlCalculatorBinomialTest, LargeTest)
{
    MdlCalculator calc;
    const double_t expected = 123447.21743029976;
    const double_t abs_error = expected * std::pow(10.0, -(DBL_DIG - 1));
    EXPECT_NEAR(expected, calc.codeBinomial(123456, 123456/2), abs_error);
}

TEST(MdlCalculatorBinomialTest, TopTriangleTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(0.0, calc.codeBinomial(1, 1));
}
