#include <gtest/gtest.h>
#include <stdexcept>
#include <limits.h>
#include "../classes/scoring/mdlcalculator.hpp"

TEST(MdlCalculatorRealTest, TooSmallPrecisionTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeReal(0.2843549532147824, -1), std::domain_error);
}

TEST(MdlCalculatorRealTest, MinPrecisionTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(6.0371347327296968, calc.codeReal(0.2843549532147824, 0));
}

TEST(MdlCalculatorRealTest, MaxPrecisionTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(69.82119766537403, calc.codeReal(0.2843549532147824, DBL_DIG));
}

TEST(MdlCalculatorRealTest, TooLargePrecisionTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeReal(0.2843549532147824, DBL_DIG + 1), std::domain_error);
}

TEST(MdlCalculatorRealTest, LargeNumberTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(20.67839097009507, calc.codeReal(1234567.89, 3));
}

TEST(MdlCalculatorRealTest, NegativeNumberTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(20.67839097009507, calc.codeReal(-1234567.89, 3));
}

TEST(MdlCalculatorRealTest, ZeroTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(5.0371347327296965, calc.codeReal(0.0, 3));
}

TEST(MdlCalculatorRealTest, SmallestPositiveNonZeroTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(5.0371347327296965, calc.codeReal(std::numeric_limits<double_t>::min(), DBL_DIG));
}

TEST(MdlCalculatorRealTest, SmallestNegativeNonZeroTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(5.0371347327296965, calc.codeReal(-std::numeric_limits<double_t>::min(), DBL_DIG));
}

TEST(MdlCalculatorRealTest, SmallAtPrecisionTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(10.446567429182842, calc.codeReal(0.00001, 1));
}

TEST(MdlCalculatorRealTest, LargeAtPrecisionTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(10.446567429182842, calc.codeReal(100000, 1));
}

TEST(MdlCalculatorRealTest, BelowNextDigitTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(23.091667639176595, calc.codeReal(9.999, 3));
}

TEST(MdlCalculatorRealTest, AtNextDigitTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(17.419748563700326, calc.codeReal(10.0, 3));
}

TEST(MdlCalculatorRealTest, AboveNextDigitTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(17.440271874222004, calc.codeReal(10.001, 3));
}
