#include <gtest/gtest.h>
#include <stdexcept>
#include "../classes/scoring/mdlcalculator.hpp"

TEST(MdlCalculatorUniformProbabilityTest, TooSmallTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeUniformProbability(-0.0001), std::domain_error);
}

TEST(MdlCalculatorUniformProbabilityTest, ZeroTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeUniformProbability(0.0), std::domain_error);
}

TEST(MdlCalculatorUniformProbabilityTest, SmallTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(11.0, calc.codeUniformProbability(1.0/2048.0));
}

TEST(MdlCalculatorUniformProbabilityTest, ValidTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(1.0, calc.codeUniformProbability(0.5));
}

TEST(MdlCalculatorUniformProbabilityTest, OneTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(0.0, calc.codeUniformProbability(1.0));
}

TEST(MdlCalculatorUniformProbabilityTest, TooLargeTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeUniformProbability(1.0001), std::domain_error);
}

TEST(MdlCalculatorUniformPossibilitiesTest, NegativeTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeUniformPossibilities(-1), std::domain_error);
}

TEST(MdlCalculatorUniformPossibilitiesTest, ZeroTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeUniformPossibilities(0), std::domain_error);
}

TEST(MdlCalculatorUniformPossibilitiesTest, OneTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(0.0, calc.codeUniformPossibilities(1));
}

TEST(MdlCalculatorUniformPossibilitiesTest, MultipleTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(1.0, calc.codeUniformPossibilities(2));
}

TEST(MdlCalculatorUniformPossibilitiesTest, MaxTest)
{
    MdlCalculator calc;
    EXPECT_DOUBLE_EQ(63.0, calc.codeUniformPossibilities(INT64_MAX));
}

TEST(MdlCalculatorUniformPossibilitiesTest, MinTest)
{
    MdlCalculator calc;
    EXPECT_THROW(calc.codeUniformPossibilities(INT64_MIN), std::domain_error);
}
