#include <gtest/gtest.h>
#include <mlpack/methods/decision_tree/decision_tree.hpp>

TEST(MLPackTest, DecisionTreeTest)
{
    arma::mat features("3.1 0.0 0.0 3.0 4.4 10.0 6.2 7.8;0 1 0 1 0 1 0 1");
    mlpack::data::DatasetInfo info(2);
    info.Type(0) = mlpack::data::Datatype::numeric;
    info.Type(1) = mlpack::data::Datatype::categorical;
    EXPECT_EQ(0, info.MapString<size_t>("red", 1));
    EXPECT_EQ(1, info.MapString<size_t>("green", 1));
    arma::Row<size_t> labels {0, 1, 0, 1, 1, 1, 1, 1};
    mlpack::DecisionTree<> myTree(features, info, labels, 2, 2, 1e-7, 3);
    // root node of tree
    ASSERT_EQ(2, myTree.NumChildren());
    EXPECT_EQ(0, myTree.SplitDimension());
    EXPECT_EQ(3.75, myTree.getClassProbabilities()[0]);
    // right branch of tree
    EXPECT_EQ(0, myTree.Child(1).NumChildren());
    EXPECT_EQ(1.0, myTree.Child(1).getClassProbabilities()[1]);
    // left branch of tree
    ASSERT_EQ(2, myTree.Child(0).NumChildren());
    EXPECT_EQ(1, myTree.Child(0).SplitDimension());
    // left-left branch of tree
    EXPECT_EQ(0, myTree.Child(0).Child(0).NumChildren());
    EXPECT_EQ(1.0, myTree.Child(0).Child(0).getClassProbabilities()[0]);
    // left-right branch of tree
    EXPECT_EQ(0, myTree.Child(0).Child(1).NumChildren());
    EXPECT_EQ(1.0, myTree.Child(0).Child(1).getClassProbabilities()[1]);
}
