#include <gtest/gtest.h>
#include "../classes/preprocessing/quantilehistogramestimator.hpp"

TEST(QuantileHistogramEstimatorTest, MoreBinsThanPossibleTest)
{
    QuantileHistogramEstimator estimator;
    std::vector<double_t> data {1, 1, 1, 1, 2, 2};
    Histogram expectedHistogram (std::vector<double_t>{0.5, 1.5, 2.5}, std::vector<int64_t>{4, 2});
    EXPECT_EQ(expectedHistogram.toString(), estimator.computeHistogram(data, 5, 1.0, 1.0).toString());
}

TEST(QuantileHistogramEstimatorTest, FewerBinsThanPossibleTest)
{
    QuantileHistogramEstimator estimator;
    std::vector<double_t> data {1, 1, 1, 1, 2, 2};
    Histogram expectedHistogram (std::vector<double_t>{0.5, 2.5}, std::vector<int64_t>{6});
    EXPECT_EQ(expectedHistogram.toString(), estimator.computeHistogram(data, 1, 1.0, 1.0).toString());
}

TEST(QuantileHistogramEstimatorTest, NoBinsTest)
{
    QuantileHistogramEstimator estimator;
    std::vector<double_t> data {1, 1, 1, 1, 2, 2};
    EXPECT_THROW(estimator.computeHistogram(data, 0, 1.0, 1.0), std::invalid_argument);
}

TEST(QuantileHistogramEstimatorTest, InvalidAccuracyTest)
{
    QuantileHistogramEstimator estimator;
    std::vector<double_t> data {1, 1, 1, 1, 2, 2};
    EXPECT_THROW(estimator.computeHistogram(data, 1, 0.0, 0.0), std::invalid_argument);
}

TEST(QuantileHistogramEstimatorTest, AllIdenticalValuesTest)
{
    QuantileHistogramEstimator estimator;
    std::vector<double_t> data {1, 1, 1, 1, 1, 1, 1, 1};
    Histogram expectedHistogram (std::vector<double_t>{0.5, 1.5}, std::vector<int64_t>{8});
    EXPECT_EQ(expectedHistogram.toString(), estimator.computeHistogram(data, 8, 1.0, 1.0).toString());
}

TEST(QuantileHistogramEstimatorTest, MinMaxOutliersTest)
{
    QuantileHistogramEstimator estimator;
    std::vector<double_t> data {-1000, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1000};
    Histogram expectedHistogram (
        std::vector<double_t>{-1000.5, 0.5, 1.5, 1000.5}, std::vector<int64_t>{1, 6, 5});
    EXPECT_EQ(expectedHistogram.toString(), estimator.computeHistogram(data, 4, 1.0, 1.0).toString());
}
