#include <gtest/gtest.h>
#include "../classes/datamodel/rulemodel.hpp"
#include "../classes/datamodel/updaterules/valuerule.hpp"

TEST(RuleModelTest, ConstructorSuccessTest)
{
    std::unique_ptr<Condition> condition = std::make_unique<Condition>();
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num");
    std::unique_ptr<UpdateRule> updateRule = std::make_unique<ValueRule>(numVariable, 1.0);
    RuleModel::RulesType rules;
    rules.push_back(std::make_unique<ModificationRule>(condition, updateRule));
    RuleModel ruleModel(rules);
    EXPECT_EQ(1, ruleModel.getRules().size());
}

TEST(RuleModelTest, ConstructorEmptySuccessTest)
{
    RuleModel::RulesType rules;
    RuleModel ruleModel(rules);
    EXPECT_EQ(0, ruleModel.getRules().size());
}

TEST(RuleModelTest, ConstructorExceptionTest)
{
    std::unique_ptr<Condition> condition = std::make_unique<Condition>();
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num");
    std::unique_ptr<UpdateRule> updateRule = std::make_unique<ValueRule>(numVariable, 1.0);
    RuleModel::RulesType rules;
    rules.push_back(std::make_unique<ModificationRule>(condition, updateRule));
    rules.push_back(nullptr);
    EXPECT_THROW(RuleModel ruleModel(rules), std::invalid_argument);
}

TEST(RuleModelTest, RulesSetSuccessTest)
{
    std::unique_ptr<Condition> condition = std::make_unique<Condition>();
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num");
    std::unique_ptr<UpdateRule> updateRule = std::make_unique<ValueRule>(numVariable, 1.0);
    RuleModel::RulesType rules;
    rules.push_back(std::make_unique<ModificationRule>(condition, updateRule));
    RuleModel ruleModel(rules);
    RuleModel::RulesType newRules;
    ruleModel.setRules(newRules);
    EXPECT_EQ(0, ruleModel.getRules().size());
}

TEST(RuleModelTest, RulesSetExceptionTest)
{
    std::unique_ptr<Condition> condition = std::make_unique<Condition>();
    std::shared_ptr<NumericalVariable> numVariable = std::make_shared<NumericalVariable>("num");
    std::unique_ptr<UpdateRule> updateRule = std::make_unique<ValueRule>(numVariable, 1.0);
    RuleModel::RulesType rules;
    rules.push_back(std::make_unique<ModificationRule>(condition, updateRule));
    RuleModel ruleModel(rules);
    RuleModel::RulesType newRules;
    newRules.push_back(nullptr);
    EXPECT_THROW(ruleModel.setRules(newRules), std::invalid_argument);
}
