#include <gtest/gtest.h>
#include "../classes/datamodel/updaterules/valuesetrule.hpp"
#include "../classes/datamodel/eventlogs/categoricalvariable.hpp"

TEST(ValueSetRuleTest, LocalObjectTest)
{
    std::set<std::string> vowelDomain = {"a", "e", "i", "o", "u"};
    std::shared_ptr<CategoricalVariable> catVariable =
        std::make_shared<CategoricalVariable>("vowel", vowelDomain);
    ValueSetRule::SetType set = {"a", "i", "o"};
    ValueSetRule rule(catVariable, set);
    EXPECT_EQ(set, rule.getSet());
    set.insert("e");
    EXPECT_NE(set, rule.getSet());
    rule.setSet(set);
    EXPECT_EQ(set, rule.getSet());
}

TEST(ValueSetRuleTest, PointerTest)
{
    std::set<std::string> vowelDomain = {"a", "e", "i", "o", "u"};
    std::shared_ptr<CategoricalVariable> catVariable =
        std::make_shared<CategoricalVariable>("vowel", vowelDomain);
    std::unique_ptr<ValueSetRule::SetType> set = std::make_unique<ValueSetRule::SetType>();
    set->insert("a");
    set->insert("i");
    set->insert("o");
    ValueSetRule rule(catVariable, *set);
    EXPECT_EQ(*set, rule.getSet());
    set->insert("e");
    EXPECT_NE(*set, rule.getSet());
    rule.setSet(*set);
    EXPECT_EQ(*set, rule.getSet());
}
