#-------------------------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See https://go.microsoft.com/fwlink/?linkid=2090316 for license information.
#-------------------------------------------------------------------------------------------------------------

FROM debian:bullseye

# Avoid warnings by switching to noninteractive
ENV DEBIAN_FRONTEND=noninteractive

# This Dockerfile adds a non-root 'vscode' user with sudo access. However, for Linux,
# this user's GID/UID must match your local user UID/GID to avoid permission issues
# with bind mounts. Update USER_UID / USER_GID if yours is not 1000. See
# https://aka.ms/vscode-remote/containers/non-root-user for details.
ARG USERNAME=vscode
ARG USER_UID=1000
ARG USER_GID=$USER_UID

# Add additional root CA if available
RUN apt-get update \
    && apt-get -y install ca-certificates \
    && mkdir -p /tmp/certificate-setup
# optionally copy the additional root CA
COPY dummycopy.txt root-ca.cr[t] /tmp/certificate-setup/
# setup the new root CA if available and cleanup
RUN cp /tmp/certificate-setup/*.crt /usr/local/share/ca-certificates/ 2>/dev/null || : \
    && update-ca-certificates \
    && rm -rf /tmp/certificate-setup
ENV NODE_EXTRA_CA_CERTS=/etc/ssl/certs/ca-certificates.crt

# Configure apt and install packages
RUN apt-get update \
    && apt-get -y install --no-install-recommends apt-utils dialog 2>&1 \
    # 
    # Verify git, process tools, lsb-release (useful for CLI installs) installed
    && apt-get -y install git iproute2 procps lsb-release \
    #
    # Install C++ tools
    && apt-get -y install build-essential cmake cppcheck valgrind \
    #
    # Create a non-root user to use if preferred - see https://aka.ms/vscode-remote/containers/non-root-user.
    && groupadd --gid $USER_GID $USERNAME \
    && useradd -s /bin/bash --uid $USER_UID --gid $USER_GID -m $USERNAME \
    # [Optional] Add sudo support for the non-root user
    && apt-get install -y sudo \
    && echo $USERNAME ALL=\(root\) NOPASSWD:ALL > /etc/sudoers.d/$USERNAME\
    && chmod 0440 /etc/sudoers.d/$USERNAME \
    #
    # Clean up
    && apt-get autoremove -y \
    && apt-get clean -y \
    && rm -rf /var/lib/apt/lists/*

# Install curl
RUN apt-get update \
    && apt-get -y install curl

# Install capnproto
RUN mkdir /tmp/capnproto-setup \
    && cd /tmp/capnproto-setup \
    && curl -O https://capnproto.org/capnproto-c++-0.10.2.tar.gz \
    && tar zxf capnproto-c++-0.10.2.tar.gz \
    && cd capnproto-c++-0.10.2 \
    && ./configure \
    && make -j6 \
    && sudo make install \
    && cd ~ \
    && rm -rf /tmp/capnproto-setup

# Install dependencies for mlpack
RUN apt-get update \
    && apt-get -y install libarmadillo-dev>=1:3.6.0 \
    && apt-get -y install libboost-dev \
    && apt-get -y install libboost-program-options-dev \
    && apt-get -y install libboost-serialization-dev \
    && apt-get -y install libboost-test-dev \
    && apt-get -y install libcereal-dev \
    && apt-get -y install liblapack-dev \
    && apt-get -y install libxml2-dev \
    && apt-get -y install python3-numpy \
    && apt-get -y install python3-numpy-abi9 \
    && apt-get -y install libensmallen-dev

# Setup SSL certs for git
RUN git config --global http.sslCAInfo /etc/ssl/certs/ca-certificates.crt

# Switch back to dialog for any ad-hoc use of apt-get
ENV DEBIAN_FRONTEND=
