FROM debian:bullseye

# Add additional root CA if available
RUN apt-get update \
    && apt-get -y install ca-certificates \
    && mkdir -p /tmp/certificate-setup
# optionally copy the additional root CA
COPY dummycopy.txt root-ca.cr[t] /tmp/certificate-setup/
# setup the new root CA if available and cleanup
RUN cp /tmp/certificate-setup/*.crt /usr/local/share/ca-certificates/ 2>/dev/null || : \
    && update-ca-certificates \
    && rm -rf /tmp/certificate-setup
ENV NODE_EXTRA_CA_CERTS=/etc/ssl/certs/ca-certificates.crt

# Install tools for capnproto
RUN apt-get update \
    && apt-get -y install git \
    && apt-get -y install build-essential \
    && apt-get -y install curl \
    && apt-get -y install sudo

# Install capnproto
RUN mkdir /tmp/capnproto-setup \
    && cd /tmp/capnproto-setup \
    && curl -O https://capnproto.org/capnproto-c++-0.10.2.tar.gz \
    && tar zxf capnproto-c++-0.10.2.tar.gz \
    && cd capnproto-c++-0.10.2 \
    && ./configure \
    && make -j6 \
    && sudo make install \
    && cd ~ \
    && rm -rf /tmp/capnproto-setup

# Install python tools and graphviz
RUN apt-get update \
    && apt-get -y install python-dev \
    && apt-get -y install pip \
    && apt-get -y install graphviz

# Install python packages
RUN pip install notebook \
    && pip install matplotlib \
    && pip install pandas \
    && pip install -U cython \
    && pip install -U setuptools \
    && pip install pycapnp \
    && pip install pm4py \
    && pip install -U scikit-learn

# Setup SSL certs for git
RUN git config --global http.sslCAInfo /etc/ssl/certs/ca-certificates.crt
