FROM python:3.7

# Add additional root CA if available
RUN apt-get update \
    && apt-get -y install ca-certificates \
    && mkdir -p /tmp/certificate-setup
# optionally copy the additional root CA
COPY dummycopy.txt root-ca.cr[t] /tmp/certificate-setup/
# setup the new root CA if available and cleanup
RUN cp /tmp/certificate-setup/*.crt /usr/local/share/ca-certificates/ 2>/dev/null || : \
    && update-ca-certificates \
    && pip config set global.cert /etc/ssl/certs/ca-certificates.crt \
    && rm -rf /tmp/certificate-setup
ENV NODE_EXTRA_CA_CERTS=/etc/ssl/certs/ca-certificates.crt

# install dependency of SSD++ (that is missing in pip)
RUN pip install numba

# install official SSD++ package
RUN pip install rulelist==0.2.0

# patch error with numpy.unique() in SSD++ package
RUN sed -i \
    's/self.categories = self.values.unique()/self.categories = np.unique(self.values)/g' \
    /usr/local/lib/python3.7/site-packages/rulelist/datastructure/attribute/nominal_attribute.py

# Install tools for capnproto
RUN apt-get update \
    && apt-get -y install git \
    && apt-get -y install build-essential \
    && apt-get -y install curl \
    && apt-get -y install sudo

# Install capnproto
RUN mkdir /tmp/capnproto-setup \
    && cd /tmp/capnproto-setup \
    && curl -O https://capnproto.org/capnproto-c++-0.10.2.tar.gz \
    && tar zxf capnproto-c++-0.10.2.tar.gz \
    && cd capnproto-c++-0.10.2 \
    && ./configure \
    && make -j6 \
    && sudo make install \
    && cd ~ \
    && rm -rf /tmp/capnproto-setup

# Install python packages
RUN pip install notebook \
    && pip install -U cython \
    && pip install -U setuptools \
    && pip install pycapnp \
    && pip install tqdm

# Setup SSL certs for git
RUN git config --global http.sslCAInfo /etc/ssl/certs/ca-certificates.crt
