# fix working directory
import os
abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

from findsubgroup_parrelle import *
from exp_utils import *
import numpy as np
import pandas as pd
import re
import sys
sys.path.append("../..")
from experiments.dataframetocapnp import dataframeFromMessage
import capnp
capnp.remove_import_hook()
interface_capnp = capnp.load("../../cpp-tool/capnproto/interface.capnp", imports=["/usr/local/include"])

# global parameters
max_num_bin_numeric = 100
flt_format = '%.3f'
max_dp = 20
append_to_current_csv = False
max_num_subgroups = 500
max_iter = 500
random_st = 1
num_cores = 5
nbeam = 5
num_initial_rule_list=5

class ServerImplementation(interface_capnp.SimpleProcessDataUpdates.Server):
    def findModel(self, log, targetVariables, **kwargs):
        # only categorical target variables are allowed
        for variable in log.variables:
            if variable.which() == "numerical" and variable.name in targetVariables:
                raise RuntimeError("TURS cannot predict numerical variables")
        
        # collect all categorical variables
        catVariables = []
        for variable in log.variables:
            if variable.which() == "categorical":
                catVariables.append(variable.name)

        # get dataframe and convert data type of categorical variables
        dataframe = dataframeFromMessage(log)
        for variable in catVariables:
            dataframe[variable] = pd.Categorical(dataframe[variable])

        # remove case ID since TURS can't use it anyways
        caseIdColumn = "case:concept:name"
        dataframe.drop(caseIdColumn, axis=1, inplace=True)

        # no given target variables => all categorical variables are considered as targets
        if len(targetVariables) == 0:
            targetVariables = sorted(catVariables)
        else:
            targetVariables = sorted(targetVariables)
        
        discoveredRules = []
        # run TURS for all possible target variables
        for target in targetVariables:
            # rearrange columns since TURS assumes the target variable to be last
            dataframe = self.moveTargetColumnToLast(dataframe, target)
            # prepare matrices for TURS
            featureNames = list(dataframe.columns)
            featureNames.remove(target)
            y = dataframe[target].cat.codes.to_numpy()
            x = np.ndarray((len(dataframe[target]), len(featureNames)), dtype=float)
            for i, feature in enumerate(featureNames):
                if feature in catVariables:
                    x[:, i] = dataframe[feature].cat.codes.to_numpy()
                else:
                    x[:, i] = dataframe[feature]
            
            # run TURS
            x_train, x_test, y_train, y_test, features, parameters = pre_run_cars(x, y,
                range(len(y)), [], max_dp, max_iter, max_num_bin_numeric, max_num_subgroups, dataframe)
            subgroup_set, _, _, _ = find_rule_set_initby_list_paralle(y_train, features, parameters,
                CLASSIFICATION, num_cores, nbeam, num_initial_rule_list)
            
            # convert TURS result to data modification rules
            discoveredRules.extend(self.subgroupSetToRules(
                subgroup_set, featureNames, dataframe, target))

        return interface_capnp.RuleModel.new_message(
            rules=discoveredRules, variables=list(log.variables))

    def moveTargetColumnToLast(self, dataframe, targetColumnName):
        columns = dataframe.columns.tolist()
        columns.remove(targetColumnName)
        columns = columns + [targetColumnName]
        return dataframe[columns]

    def subgroupSetToRules(self, subgroupSet, featureNames, dataframe, targetName):
        rules = []
        for subgroup in subgroupSet:
            # construct condition
            elements = []
            for i, textCondition in enumerate(subgroup.rule):
                # convert condition format
                self.extractCategoricalCondition(elements, textCondition, featureNames, dataframe)
                self.extractNumericalCondition(elements, textCondition, featureNames, dataframe)

                # connect multiple conditions with an AND
                if len(subgroup.rule) > 1 and i >= 1:
                    element = interface_capnp.ConditionElement.new_message()
                    element.andOperator = None
                    elements.append(element)

            # insert condition into rule
            rule = interface_capnp.ModificationRule.new_message()
            rule.condition.init("elements", len(elements))
            for i, element in enumerate(elements):
                rule.condition.elements[i] = element

            # construct update rule
            predictionIndex = np.argmax(np.bincount(dataframe[targetName][subgroup.indices].cat.codes))
            prediction = pd.Categorical.from_codes(
                [predictionIndex], dtype=dataframe[targetName].dtype)[0]
            rule.updateRule.variable = targetName
            rule.updateRule.init("valueSetRule")
            rule.updateRule.valueSetRule.init("set", 1)
            rule.updateRule.valueSetRule.set[0] = str(prediction)

            # add rule to result
            rules.append(rule)
        return rules

    def extractCategoricalCondition(self, conditionElements, textCondition, featureNames, dataframe):
        # get variable index
        variableSearch = re.search(r"X(\d+)in\(", textCondition)
        if variableSearch == None: # condition is not categorical => skip
            return
        conditionVariableIndex = int(variableSearch.group(1))
        
        # add atomic conditions for all elements in the set
        valueIndices = [int(x) for x in re.findall(r"(\d+)[,\)]", textCondition)]
        for j, valueIndex in enumerate(valueIndices):
            element = interface_capnp.ConditionElement.new_message()
            element.init("atomicCondition")
            variableName = featureNames[conditionVariableIndex]
            element.atomicCondition.variable = variableName
            element.atomicCondition.init("categoricalValueCondition")
            value = pd.Categorical.from_codes(
                [valueIndex], dtype=dataframe[variableName].dtype)[0]
            element.atomicCondition.categoricalValueCondition.value = str(value)
            conditionElements.append(element)

            # connect elements of the set with an OR
            if len(valueIndices) > 1 and j >= 1:
                element = interface_capnp.ConditionElement.new_message()
                element.orOperator = None
                conditionElements.append(element)

    def extractNumericalCondition(self, conditionElements, textCondition, featureNames, dataframe):
        # extract information from given text
        formatMatch = re.match(r"X(\d+)(<=|>)(\d+\.?\d*)", textCondition)
        if formatMatch == None: # condition is not numerical => skip
            return
        conditionVariableIndex = int(formatMatch.group(1))
        numericalBound = float(formatMatch.group(3))

        # build condition object
        element = interface_capnp.ConditionElement.new_message()
        element.init("atomicCondition")
        variableName = featureNames[conditionVariableIndex]
        element.atomicCondition.variable = variableName
        if formatMatch.group(2) == "<=":
            element.atomicCondition.init("upperBoundCondition")
            element.atomicCondition.upperBoundCondition.value = numericalBound
        else:
            element.atomicCondition.init("lowerBoundCondition")
            element.atomicCondition.lowerBoundCondition.value = numericalBound

        # add condition object
        conditionElements.append(element)



if __name__ == "__main__":
    # start server
    server = capnp.TwoPartyServer("*:21653", bootstrap=ServerImplementation())
    server.run_forever()
