import os
import shutil

# fix working directory
import os
abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

import capnp
capnp.remove_import_hook()
interface_capnp = capnp.load("../../cpp-tool/capnproto/interface.capnp", imports=["/usr/local/include"])

import sys
sys.path.append("../..")
from experiments.modelgeneration import ModelGenerator

# connect to my server
client = capnp.TwoPartyClient("vr-8:21654")
processDataUpdatesCpp = client.bootstrap().cast_as(interface_capnp.ProcessDataUpdates)

modelGen = ModelGenerator(interface_capnp, processDataUpdatesCpp, None)

# ==================== CATEGORICAL TARGETS ====================
# clean start for the noise experiment
experimentName = "hyperparameterCategorical"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=20, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="categorical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models
modelGen.generateHyperparameterModels(experimentName, trueModels,
                                      [0.1, 0.2], [2, 10, 30, 50, 70], 5,
                                      runtimesFilename=experimentName + "/runtimes.csv")

# ==================== NUMERICAL TARGETS ====================
# clean start for the noise experiment
experimentName = "hyperparameterNumerical"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=20, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="numerical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models
modelGen.generateHyperparameterModels(experimentName, trueModels,
                                      [0.1, 0.2], [2, 10, 30, 50, 70], 5,
                                      runtimesFilename=experimentName + "/runtimes.csv")
