import os
import shutil

# fix working directory
import os
abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

import capnp
capnp.remove_import_hook()
interface_capnp = capnp.load("../../cpp-tool/capnproto/interface.capnp", imports=["/usr/local/include"])

import sys
sys.path.append("../..")
from experiments.modelgeneration import ModelGenerator

# connect to my server
client = capnp.TwoPartyClient("vr-7:21654")
processDataUpdatesCpp = client.bootstrap().cast_as(interface_capnp.ProcessDataUpdates)

# connect to SSD++ server
client = capnp.TwoPartyClient("vr-7:21652")
processDataUpdatesSsdpp = client.bootstrap().cast_as(interface_capnp.SimpleProcessDataUpdates)

modelGen = ModelGenerator(interface_capnp, processDataUpdatesCpp, processDataUpdatesSsdpp)

# ==================== CATEGORICAL TARGETS ====================
# clean start for the noise experiment
experimentName = "ssdppNoiseVariationCategorical"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=10, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="categorical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models with varying log lengths
modelGen.generateModelEstimates(experimentName, trueModels, 
                                [0.0, 0.1, 0.2, 0.3], [2000], 10, "ssdpp")

# clean start for the sample experiment
experimentName = "ssdppSampleVariationCategorical"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=10, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="categorical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models with varying log lengths
modelGen.generateModelEstimates(experimentName, trueModels, [0.1],
                                [500, 750, 1000, 1500, 2000], 10, "ssdpp")

# ==================== NUMERICAL TARGETS ====================
# clean start for the noise experiment
experimentName = "ssdppNoiseVariationNumerical"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=10, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="numerical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models with varying log lengths
modelGen.generateModelEstimates(experimentName, trueModels, 
                                [0.0, 0.1, 0.2, 0.3], [2000], 10, "ssdpp")

# clean start for the sample experiment
experimentName = "ssdppSampleVariationNumerical"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=10, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="numerical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models with varying log lengths
modelGen.generateModelEstimates(experimentName, trueModels, [0.1],
                                [500, 750, 1000, 1500, 2000], 10, "ssdpp")
