import os
import shutil

# fix working directory
import os
abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

import capnp
capnp.remove_import_hook()
interface_capnp = capnp.load("../../cpp-tool/capnproto/interface.capnp", imports=["/usr/local/include"])

import sys
sys.path.append("../..")
from experiments.modelgeneration import ModelGenerator

# connect to my server
client = capnp.TwoPartyClient("vr-8:21654")
processDataUpdatesCpp = client.bootstrap().cast_as(interface_capnp.ProcessDataUpdates)

# connect to TURS server
client = capnp.TwoPartyClient("vr-8:21653")
processDataUpdatesTurs = client.bootstrap().cast_as(interface_capnp.SimpleProcessDataUpdates)

# connect to SSD++ server
client = capnp.TwoPartyClient("vr-8:21652")
processDataUpdatesSsdpp = client.bootstrap().cast_as(interface_capnp.SimpleProcessDataUpdates)

# ==================== CATEGORICAL TARGETS ====================
modelGen = ModelGenerator(interface_capnp, processDataUpdatesCpp, {
    "turs": processDataUpdatesTurs,
    "ssdpp": processDataUpdatesSsdpp,
    "tree": None
})

# clean start for the experiment
experimentName = "scalingExperiment20ModelsCat"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=20, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="categorical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models with varying log lengths
modelGen.generateModelEstimates(experimentName, trueModels, [0.1],
                                [250, 500, 750, 1000, 1250, 1500], 5,
                                runtimesFilename=experimentName + "/runtimes.csv")

# ==================== NUMERICAL TARGETS ====================
modelGen = ModelGenerator(interface_capnp, processDataUpdatesCpp,
                          {"ssdpp": processDataUpdatesSsdpp,
                           "tree": None})

# clean start for the experiment
experimentName = "scalingExperiment20ModelsNum"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels(numberOfModels=20, numberOfRules=5, catVariablesCount=2, 
                                         numVariablesCount=2, valuesPerCatVariable=4, targetType="numerical")
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models with varying log lengths
modelGen.generateModelEstimates(experimentName, trueModels, [0.1],
                                [250, 500, 750, 1000, 1250, 1500], 5,
                                runtimesFilename=experimentName + "/runtimes.csv")
