import os
import shutil

# fix working directory
import os
abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

import capnp
capnp.remove_import_hook()
interface_capnp = capnp.load("../../cpp-tool/capnproto/interface.capnp", imports=["/usr/local/include"])

import sys
sys.path.append("../..")
from experiments.modelgeneration import ModelGenerator

# connect to my server
client = capnp.TwoPartyClient("vr-1:21654")
processDataUpdatesCpp = client.bootstrap().cast_as(interface_capnp.ProcessDataUpdates)

# connect to TURS server
client = capnp.TwoPartyClient("vr-1:21653")
processDataUpdatesTurs = client.bootstrap().cast_as(interface_capnp.SimpleProcessDataUpdates)

modelGen = ModelGenerator(interface_capnp, processDataUpdatesCpp, processDataUpdatesTurs)

# clean start for the experiment
experimentName = "tursSampleVariation"
shutil.rmtree(experimentName, ignore_errors=True)
os.mkdir(experimentName)

# generate and save ground-truth models
trueModels = modelGen.generateTrueModels()
modelGen.saveTrueModels(experimentName, trueModels)
modelGen.saveEmptyModels(experimentName, trueModels)

# estimate models with varying log lengths
modelGen.generateModelEstimates(experimentName, trueModels, [0.1], [50, 100, 200, 300, 400, 500], 10)
