import numpy as np
import pandas as pd
from .dataframetocapnp import dataframeFromMessage
from sklearn.model_selection import GridSearchCV
from sklearn.tree import DecisionTreeClassifier
from sklearn.tree import DecisionTreeRegressor

class DecisionTreeAdapter:
    def eventLogToDecisionTreeDataset(self, eventLogMessage, targetColumn):
        catLabelMapping = None
        # prepare dataset for decision trees
        dataframe = dataframeFromMessage(eventLogMessage, disableProgressBar=True)
        dataframe.drop("case:concept:name", axis="columns", inplace=True)
        # make categorical columns actually categorical
        for column in dataframe.columns:
            if dataframe[column].dtype != "float64":
                dataframe[column] = dataframe[column].astype("category")
        # create numpy array from all feature variables
        X = np.zeros((len(dataframe), 0))
        for inputColumn in dataframe.columns:
            if inputColumn == targetColumn:
                continue
            if dataframe[inputColumn].dtype == "float64":
                X = np.hstack((X, dataframe[inputColumn].to_numpy()[:, np.newaxis]))
            else:
                # one hot encoding for categorical input variables
                X = np.hstack((X, pd.get_dummies(dataframe[inputColumn]).to_numpy()))
        # create numpy array for target variable
        if dataframe[targetColumn].dtype == "float64":
            y = dataframe[targetColumn].to_numpy()
        else:
            y = dataframe[targetColumn].cat.codes.to_numpy(dtype="int64")
            catLabelMapping = dict(enumerate(dataframe[targetColumn].cat.categories))

        return X, y, catLabelMapping
    
    def findDecisionTreeModel(self, eventLogMessage, targetVariables):
        from .evaluation import getVariableType
        result = dict()
        for targetVariableName in targetVariables:
            X, y, _ = self.eventLogToDecisionTreeDataset(eventLogMessage, targetVariableName)
            if getVariableType(targetVariableName, eventLogMessage.variables) == "numerical":
                # learn regressor for numerical targets
                tree = self.getBestCV(X, y, DecisionTreeRegressor(), "neg_root_mean_squared_error")
            elif getVariableType(targetVariableName, eventLogMessage.variables) == "categorical":
                # learn classifier for categorical targets
                tree = self.getBestCV(X, y, DecisionTreeClassifier(), "f1_macro")
            # create dictionary of the decision trees for different target variables
            result[targetVariableName] = tree
        return result
    
    def getBestCV(self, X, y, estimator, scoring, max_depth = 7):
        parameters = {"max_depth": list(range(1, max_depth))}
        gridSearch = GridSearchCV(estimator, parameters, scoring=scoring)
        gridSearch.fit(X, y)
        return gridSearch.best_estimator_
        