#!/usr/bin/env python
# coding: utf-8
import pm4py
import pandas as pd

# fix working directory
import os

abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

# load and preprocess
log = pm4py.read_xes("../datasets/Sepsis Cases - Event Log.xes")
log.sort_values("time:timestamp", axis="rows", inplace=True)
log.drop(["lifecycle:transition", "time:timestamp"], axis="columns", inplace=True)

# convert to Cap'n Proto Message
import capnp
capnp.remove_import_hook()
interface_capnp = capnp.load("../../cpp-tool/capnproto/interface.capnp", imports=["/usr/local/include"])

import sys
sys.path.append("../..")
from experiments.dataframetocapnp import dataframeToMessage
eventLogMessage = dataframeToMessage(log, fillMissingValues=True)

# connect to server
client = capnp.TwoPartyClient("vr-1:21654")
processDataUpdates = client.bootstrap().cast_as(interface_capnp.ProcessDataUpdates)

# compute model of data modification rules
from datetime import datetime
startTime = datetime.now()
modelPromise = processDataUpdates.findModel(eventLogMessage)
foundModel = modelPromise.wait().model
print(processDataUpdates.modelToString(foundModel).wait().string)
print("Execution time:", datetime.now() - startTime)
