import capnp
import codecs
import json
import matplotlib.pyplot as plt
import subprocess

def computeScoreDevelopment(modelFileName, log, interface_capnp, forward=False):

    # connect to RPC interface
    client = capnp.TwoPartyClient("cpp-tool:21654")
    processDataUpdates = client.bootstrap().cast_as(interface_capnp.ProcessDataUpdates)

    # get number of rules in the model
    with open("/tmp/" + modelFileName + ".bin", "rb") as file:
        fullModel = interface_capnp.RuleModel.read(file)
    totalNumberOfRules = len(fullModel.rules)

    numberOfRules = []
    scores = []
    for rulesToRemove in range(totalNumberOfRules + 1):
        with codecs.open(modelFileName + ".json", "r+", encoding="utf-8") as file:
            # load the full model
            data = json.load(file)
            # remove rules
            if forward:
                # remove last rules
                del data["rules"][(len(data["rules"]) - rulesToRemove):]
            else:
                # remove first rules
                del data["rules"][:rulesToRemove]
        # write the partial model
        with codecs.open("/tmp/tempModel.json", "w+", encoding="utf-8") as file:
            json.dump(data, file, indent=4)
        
        # convert json to binary
        subprocess.run("capnp convert json:binary ../cpp-tool/capnproto/interface.capnp RuleModel" +
            " < /tmp/tempModel.json > /tmp/tempModel.bin", shell=True)
        # load partial model
        with open("/tmp/tempModel.bin", "rb") as file:
            tempModel = interface_capnp.RuleModel.read(file)
        
        # get MDL score
        scorePromise = processDataUpdates.calculateMdlScore(log=log, model=tempModel)
        score = scorePromise.wait().result

        # save values in lists
        scores.append(score)
        numberOfRules.append(totalNumberOfRules - rulesToRemove)
    
    return numberOfRules, scores

def visualizeScoreDevelopment(modelFileName, log, interface_capnp, forward=False):
    subprocess.run("capnp convert json:binary ../cpp-tool/capnproto/interface.capnp RuleModel < "
        + modelFileName + ".json > /tmp/" + modelFileName + ".bin", shell=True)
    numberOfRules, scores = computeScoreDevelopment(modelFileName, log, interface_capnp, forward)
    plt.plot(numberOfRules, scores, marker="o")
    plt.ylabel("Total encoding length [bits]")
    plt.xlabel("Numer of rules in the model")
    plt.show()