import statistics
import matplotlib.pyplot as plt

def collectEntries(eventLog, variableName, type="value"):
    entries = []
    for trace in eventLog.traces:
        for column in trace.columns:
            if column.variableName == variableName:
                columnType = column.which()
                if columnType == "categorical":
                    entries.extend([x for x in column.categorical.entries])
                elif columnType == "numerical":
                    if type == "value":
                        entries.extend([x for x in column.numerical.entries])
                    elif type == "difference":
                        traceEntries = [x for x in column.numerical.entries]
                        entries.extend([x - xprev for x, xprev in zip(
                            traceEntries[1:], traceEntries[:-1])])
                    elif type == "factor":
                        traceEntries = [x for x in column.numerical.entries if x != 0]
                        entries.extend([x / xprev for x, xprev in zip(
                            traceEntries[1:], traceEntries[:-1])])
    return entries

def visualizeVariableDistribution(eventLog, variableName, type="value", bins=50):
    entries = collectEntries(eventLog, variableName, type)
    print("Mode of the distribution:", statistics.mode(entries))
    plt.hist(entries, bins=bins)
    plt.ylabel("Number of entries")
    plt.xlabel(type + "s of variable \"" + variableName + "\"")
